#!/usr/bin/env python
# -*- coding: utf-8 -*-
import pytest

from textwrap import dedent
from flask import Flask, jsonify


pytest_plugins = 'pytester'


@pytest.fixture
def app():
    app = Flask(__name__)
    app.config['SECRET_KEY'] = '42'

    @app.route('/')
    def index():
        return app.response_class('OK')

    @app.route('/ping')
    def ping():
        return jsonify(ping='pong')

    return app


@pytest.fixture
def appdir(testdir):
    app_root = testdir.tmpdir
    test_root = app_root.mkdir('tests')

    def create_test_module(code, filename='test_app.py'):
        f = test_root.join(filename)
        f.write(dedent(code), ensure=True)
        return f

    testdir.create_test_module = create_test_module

    testdir.create_test_module('''
        import pytest

        from flask import Flask

        @pytest.fixture
        def app():
            app = Flask(__name__)
            return app
    ''', filename='conftest.py')
    return testdir



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

import pytest
try:
    from urllib2 import urlopen
except ImportError:
    from urllib.request import urlopen

from flask import url_for


pytestmark = pytest.mark.skipif(not hasattr(os, 'fork'), reason='needs fork')


class TestLiveServer:

    def test_server_is_alive(self, live_server):
        assert live_server._process
        assert live_server._process.is_alive()

    def test_server_url(self, live_server):
        assert live_server.url() == 'http://localhost:%d' % live_server.port
        assert live_server.url('/ping') == \
            'http://localhost:%d/ping' % live_server.port

    def test_server_listening(self, live_server):
        res = urlopen(live_server.url('/ping'))
        assert res.code == 200
        assert b'pong' in res.read()

    def test_url_for(self, live_server):
        assert url_for('ping', _external=True) == \
            'http://localhost:%s/ping' % live_server.port

    def test_set_application_server_name(self, live_server):
        assert live_server.app.config['SERVER_NAME'] == \
            'localhost:%d' % live_server.port

    @pytest.mark.options(server_name='example.com:5000')
    def test_rewrite_application_server_name(self, live_server):
        assert live_server.app.config['SERVER_NAME'] == \
            'example.com:%d' % live_server.port

    def test_prevent_starting_live_server(self, appdir):
        appdir.create_test_module('''
            import pytest

            def test_a(live_server):
                assert live_server._process is None
        ''')

        result = appdir.runpytest('-v', '--no-start-live-server')
        result.stdout.fnmatch_lines(['*PASSED*'])
        assert result.ret == 0

    def test_start_live_server(self, appdir):
        appdir.create_test_module('''
            import pytest

            def test_a(live_server):
                assert live_server._process
                assert live_server._process.is_alive()
        ''')
        result = appdir.runpytest('-v', '--start-live-server')
        result.stdout.fnmatch_lines(['*PASSED*'])
        assert result.ret == 0

    @pytest.mark.parametrize('clean_stop', [True, False])
    def test_clean_stop_live_server(self, appdir, monkeypatch, clean_stop):
        """Ensure the fixture is trying to cleanly stop the server.

        Because this is tricky to test, we are checking that the _stop_cleanly() internal
        function was called and reported success.
        """
        from pytest_flask.fixtures import LiveServer

        original_stop_cleanly_func = LiveServer._stop_cleanly

        stop_cleanly_result = []

        def mocked_stop_cleanly(*args, **kwargs):
            result = original_stop_cleanly_func(*args, **kwargs)
            stop_cleanly_result.append(result)
            return result

        monkeypatch.setattr(LiveServer, '_stop_cleanly', mocked_stop_cleanly)

        appdir.create_test_module('''
            import pytest
            try:
                from urllib2 import urlopen
            except ImportError:
                from urllib.request import urlopen

            from flask import url_for

            def test_a(live_server):
                @live_server.app.route('/')
                def index():
                    return 'got it', 200

                live_server.start()

                res = urlopen(url_for('index', _external=True))
                assert res.code == 200
                assert b'got it' in res.read()
        ''')
        args = [] if clean_stop else ['--no-live-server-clean-stop']
        result = appdir.runpytest_inprocess('-v', '--no-start-live-server',
                                            *args)
        result.stdout.fnmatch_lines('*1 passed*')
        if clean_stop:
            assert stop_cleanly_result == [True]
        else:
            assert stop_cleanly_result == []

    def test_add_endpoint_to_live_server(self, appdir):
        appdir.create_test_module('''
            import pytest
            try:
                from urllib2 import urlopen
            except ImportError:
                from urllib.request import urlopen

            from flask import url_for

            def test_a(live_server):
                @live_server.app.route('/new-endpoint')
                def new_endpoint():
                    return 'got it', 200

                live_server.start()

                res = urlopen(url_for('new_endpoint', _external=True))
                assert res.code == 200
                assert b'got it' in res.read()
        ''')
        result = appdir.runpytest('-v', '--no-start-live-server')
        result.stdout.fnmatch_lines(['*PASSED*'])
        assert result.ret == 0

    def test_concurrent_requests_to_live_server(self, appdir):
        appdir.create_test_module('''
            import pytest
            try:
                from urllib2 import urlopen
            except ImportError:
                from urllib.request import urlopen

            from flask import url_for

            def test_concurrent_requests(live_server):
                @live_server.app.route('/one')
                def one():
                    res = urlopen(url_for('two', _external=True))
                    return res.read()

                @live_server.app.route('/two')
                def two():
                    return '42'

                live_server.start()

                res = urlopen(url_for('one', _external=True))
                assert res.code == 200
                assert b'42' in res.read()
        ''')
        result = appdir.runpytest('-v', '--no-start-live-server')
        result.stdout.fnmatch_lines(['*PASSED*'])
        assert result.ret == 0

    @pytest.mark.parametrize('port', [5000, 5001])
    def test_live_server_fixed_port(self, port, appdir):
        appdir.create_test_module('''
            import pytest

            def test_port(live_server):
                assert live_server.port == %d
        ''' % port)
        result = appdir.runpytest('-v', '--live-server-port', str(port))
        result.stdout.fnmatch_lines(['*PASSED*'])
        assert result.ret == 0

    @pytest.mark.parametrize('host', ['127.0.0.1', '0.0.0.0'])
    def test_live_server_fixed_host(self, host, appdir):
        appdir.create_test_module('''
            import pytest

            def test_port(live_server):
                assert live_server.host == '%s'
        ''' % host)
        result = appdir.runpytest('-v', '--live-server-host', str(host))
        result.stdout.fnmatch_lines(['*PASSED*'])
        assert result.ret == 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import time
import multiprocessing
import pytest
import socket
import signal
import os
import logging

try:
    from urllib2 import URLError, urlopen
except ImportError:
    from urllib.error import URLError
    from urllib.request import urlopen

from flask import _request_ctx_stack


@pytest.yield_fixture
def client(app):
    """A Flask test client. An instance of :class:`flask.testing.TestClient`
    by default.
    """
    with app.test_client() as client:
        yield client


@pytest.fixture
def client_class(request, client):
    """Uses to set a ``client`` class attribute to current Flask test client::

        @pytest.mark.usefixtures('client_class')
        class TestView:

            def login(self, email, password):
                credentials = {'email': email, 'password': password}
                return self.client.post(url_for('login'), data=credentials)

            def test_login(self):
                assert self.login('foo@example.com', 'pass').status_code == 200

    """
    if request.cls is not None:
        request.cls.client = client


class LiveServer(object):
    """The helper class uses to manage live server. Handles creation and
    stopping application in a separate process.

    :param app: The application to run.
    :param host: The host where to listen (default localhost).
    :param port: The port to run application.
    """

    def __init__(self, app, host, port, clean_stop=False):
        self.app = app
        self.port = port
        self.host = host
        self.clean_stop = clean_stop
        self._process = None

    def start(self):
        """Start application in a separate process."""
        def worker(app, host, port):
            app.run(host=host, port=port, use_reloader=False, threaded=True)
        self._process = multiprocessing.Process(
            target=worker,
            args=(self.app, self.host, self.port)
        )
        self._process.start()

        # We must wait for the server to start listening with a maximum
        # timeout of 5 seconds.
        timeout = 5
        while timeout > 0:
            time.sleep(1)
            try:
                urlopen(self.url())
                timeout = 0
            except URLError:
                timeout -= 1

    def url(self, url=''):
        """Returns the complete url based on server options."""
        return 'http://%s:%d%s' % (self.host, self.port, url)

    def stop(self):
        """Stop application process."""
        if self._process:
            if self.clean_stop and self._stop_cleanly():
                return
            if self._process.is_alive():
                # If it's still alive, kill it
                self._process.terminate()

    def _stop_cleanly(self, timeout=5):
        """Attempts to stop the server cleanly by sending a SIGINT signal and waiting for
        ``timeout`` seconds.

        :return: True if the server was cleanly stopped, False otherwise.
        """
        try:
            os.kill(self._process.pid, signal.SIGINT)
            self._process.join(timeout)
            return True
        except Exception as ex:
            logging.error('Failed to join the live server process: %r', ex)
            return False

    def __repr__(self):
        return '<LiveServer listening at %s>' % self.url()


def _rewrite_server_name(server_name, new_port):
    """Rewrite server port in ``server_name`` with ``new_port`` value."""
    sep = ':'
    if sep in server_name:
        server_name, port = server_name.split(sep, 1)
    return sep.join((server_name, new_port))


@pytest.fixture(scope='function')
def live_server(request, app, monkeypatch, pytestconfig):
    """Run application in a separate process.

    When the ``live_server`` fixture is applied, the ``url_for`` function
    works as expected::

        def test_server_is_up_and_running(live_server):
            index_url = url_for('index', _external=True)
            assert index_url == 'http://localhost:5000/'

            res = urllib2.urlopen(index_url)
            assert res.code == 200

    """
    port = pytestconfig.getvalue('live_server_port')

    if port == 0:
        # Bind to an open port
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.bind(('', 0))
        port = s.getsockname()[1]
        s.close()

    host = pytestconfig.getvalue('live_server_host')

    # Explicitly set application ``SERVER_NAME`` for test suite
    # and restore original value on test teardown.
    server_name = app.config['SERVER_NAME'] or 'localhost'
    monkeypatch.setitem(app.config, 'SERVER_NAME',
                        _rewrite_server_name(server_name, str(port)))

    clean_stop = request.config.getvalue('live_server_clean_stop')
    server = LiveServer(app, host, port, clean_stop)
    if request.config.getvalue('start_live_server'):
        server.start()

    request.addfinalizer(server.stop)
    return server


@pytest.fixture
def config(app):
    """An application config."""
    return app.config


@pytest.fixture
def request_ctx(app):
    """The request context which contains all request relevant information,
    e.g. `session`, `g`, `flashes`, etc.
    """
    return _request_ctx_stack.top


@pytest.fixture(params=['application/json', 'text/html'])
def mimetype(request):
    return request.param


def _make_accept_header(mimetype):
    return [('Accept', mimetype)]


@pytest.fixture
def accept_mimetype(mimetype):
    return _make_accept_header(mimetype)


@pytest.fixture
def accept_json(request):
    return _make_accept_header('application/json')


@pytest.fixture
def accept_jsonp():
    return _make_accept_header('application/json-p')


@pytest.fixture(params=['*', '*/*'])
def accept_any(request):
    return _make_accept_header(request.param)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import pytest

from flask import Flask


@pytest.fixture(scope='session')
def app():
    app = Flask(__name__)
    app.config['SECRET_KEY'] = '42'
    return app


class TestOptionMarker:

    @pytest.mark.options(debug=False)
    def test_not_debug_app(self, app):
        assert not app.debug, 'Ensure the app not in debug mode'

    @pytest.mark.options(foo=42)
    def test_update_application_config(self, request, app, config):
        assert config['FOO'] == 42

    def test_application_config_teardown(self, config):
        assert 'FOO' not in config



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
pytest-flask
============

A set of `pytest <https://docs.pytest.org>`_ fixtures to test Flask
extensions and applications.

Features
--------

Plugin provides some fixtures to simplify app testing:

- ``client`` - an instance of ``app.test_client``,
- ``client_class`` - ``client`` fixture for class-based tests,
- ``config`` - the application config,
- ``live_server`` - runs an application in the background (useful for tests
  with `Selenium <http://www.seleniumhq.org>`_ and other headless browsers),
- ``request_ctx`` - the request context,
- ``accept_json``, ``accept_jsonp``, ``accept_any`` - accept headers
  suitable to use as parameters in ``client``.

To pass options to your application use the ``pytest.mark.options`` marker:

.. code:: python

    @pytest.mark.options(debug=False)
    def test_app(app):
      assert not app.debug, 'Ensure the app not in debug mode'

During tests execution the request context has been pushed, e.g. ``url_for``,
``session`` and other context bound objects are available without context
managers:

.. code:: python

    def test_app(client):
        assert client.get(url_for('myview')).status_code == 200

Response object has a ``json`` property to test a view that returns
a JSON response:

.. code:: python

    @api.route('/ping')
    def ping():
        return jsonify(ping='pong')

    def test_api_ping(client):
        res = client.get(url_for('api.ping'))
        assert res.json == {'ping': 'pong'}

If you want your tests done via Selenium or other headless browser use
the ``live_server`` fixture. The server’s URL can be retrieved using
the ``url_for`` function:

.. code:: python

    from flask import url_for

    @pytest.mark.usefixtures('live_server')
    class TestLiveServer:

        def test_server_is_up_and_running(self):
            res = urllib2.urlopen(url_for('index', _external=True))
            assert b'OK' in res.read()
            assert res.code == 200

Quick Start
-----------

To start using a plugin define your application fixture in ``conftest.py``:

.. code:: python

    from myapp import create_app

    @pytest.fixture
    def app():
        app = create_app()
        return app

Install the extension with dependencies and run your test suite:

.. code:: bash

    $ pip install pytest-flask
    $ py.test

Documentation
-------------

The latest documentation is available at
http://pytest-flask.readthedocs.org/en/latest/.

Contributing
------------

Don’t hesitate to create a `GitHub issue
<https://github.com/vitalk/pytest-flask/issues>`_ for any **bug** or
**suggestion**.

"""
import io
import os
import re
from setuptools import setup
from setuptools import find_packages


def read(*parts):
    """Reads the content of the file located at path created from *parts*."""
    try:
        return io.open(os.path.join(*parts), 'r', encoding='utf-8').read()
    except IOError:
        return ''


requirements = read('requirements', 'main.txt').splitlines() + ['pytest']
tests_require = []

extras_require = {
    'docs': read('requirements', 'docs.txt').splitlines(),
    'tests': tests_require
}


setup(
    name='pytest-flask',

    # Versions should comply with PEP440, and automatically obtained from tags
    # thanks to setuptools_scm
    use_scm_version={"write_to": "pytest_flask/_version.py"},
    setup_requires=["setuptools-scm"],

    author='Vital Kudzelka',
    author_email='vital.kudzelka@gmail.com',

    url='https://github.com/vitalk/pytest-flask',
    project_urls={
        "Source": "https://github.com/pytest-dev/pytest-flask",
        "Tracker": "https://github.com/pytest-dev/pytest-flask/issues",
    },
    description='A set of py.test fixtures to test Flask applications.',
    long_description=__doc__,
    license='MIT',

    packages=find_packages(exclude=['docs', 'tests']),
    zip_safe=False,
    platforms='any',
    install_requires=requirements,
    tests_require=tests_require,
    extras_require=extras_require,

    keywords='pytest flask testing',
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Plugins',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development :: Testing',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    # The following makes a plugin available to pytest
    entry_points={
        'pytest11': [
            'flask = pytest_flask.plugin',
        ]
    },
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    A py.test plugin which helps testing Flask applications.

    :copyright: (c) by Vital Kudzelka
    :license: MIT
"""
import sys

import pytest

from flask import json
from werkzeug.utils import cached_property

from .fixtures import (
    client, config, accept_json, accept_jsonp, accept_any, accept_mimetype,
    client_class, live_server, request_ctx
)
from .pytest_compat import getfixturevalue


class JSONResponse(object):
    """Mixin with testing helper methods for JSON responses."""

    @cached_property
    def json(self):
        """Try to deserialize response data (a string containing a valid JSON
        document) to a Python object by passing it to the underlying
        :mod:`flask.json` module.
        """
        return json.loads(self.data)

    def __eq__(self, other):
        if isinstance(other, int):
            return self.status_code == other
        # even though the Python 2-specific code works on Python 3, keep the two versions
        # separate so we can simplify the code once Python 2 support is dropped
        if sys.version_info[0] == 2:
            try:
                super_eq = super(JSONResponse, self).__eq__
            except AttributeError:
                return NotImplemented
            else:
                return super_eq(other)
        else:
            return super(JSONResponse, self).__eq__(other)

    def __ne__(self, other):
        return not self == other


def pytest_assertrepr_compare(op, left, right):
    if isinstance(left, JSONResponse) and op == '==' and isinstance(right, int):
        return [
            'Mismatch in status code for response: {} != {}'.format(
                left.status_code,
                right,
            ),
            'Response status: {}'.format(left.status),
        ]
    return None


def _make_test_response_class(response_class):
    """Extends the response class with special attribute to test JSON
    responses. Don't override user-defined `json` attribute if any.

    :param response_class: An original response class.
    """
    if 'json' in response_class.__dict__:
        return response_class

    return type(str(JSONResponse), (response_class, JSONResponse), {})


@pytest.fixture(autouse=True)
def _monkeypatch_response_class(request, monkeypatch):
    """Set custom response class before test suite and restore the original
    after. Custom response has `json` property to easily test JSON responses::

        @app.route('/ping')
        def ping():
            return jsonify(ping='pong')

        def test_json(client):
            res = client.get(url_for('ping'))
            assert res.json == {'ping': 'pong'}

    """
    if 'app' not in request.fixturenames:
        return

    app = getfixturevalue(request, 'app')
    monkeypatch.setattr(app, 'response_class',
                        _make_test_response_class(app.response_class))


@pytest.fixture(autouse=True)
def _push_request_context(request):
    """During tests execution request context has been pushed, e.g. `url_for`,
    `session`, etc. can be used in tests as is::

        def test_app(app, client):
            assert client.get(url_for('myview')).status_code == 200

    """
    if 'app' not in request.fixturenames:
        return

    app = getfixturevalue(request, 'app')

    # Get application bound to the live server if ``live_server`` fixture
    # is applied. Live server application has an explicit ``SERVER_NAME``,
    # so ``url_for`` function generates a complete URL for endpoint which
    # includes application port as well.
    if 'live_server' in request.fixturenames:
        app = getfixturevalue(request, 'live_server').app

    ctx = app.test_request_context()
    ctx.push()

    def teardown():
        ctx.pop()

    request.addfinalizer(teardown)


@pytest.fixture(autouse=True)
def _configure_application(request, monkeypatch):
    """Use `pytest.mark.options` decorator to pass options to your application
    factory::

        @pytest.mark.options(debug=False)
        def test_something(app):
            assert not app.debug, 'the application works not in debug mode!'

    """
    if 'app' not in request.fixturenames:
        return

    app = getfixturevalue(request, 'app')
    for options in request.node.iter_markers('options'):
        for key, value in options.kwargs.items():
            monkeypatch.setitem(app.config, key.upper(), value)


def pytest_addoption(parser):
    group = parser.getgroup('flask')
    group.addoption('--start-live-server',
                    action="store_true", dest="start_live_server", default=True,
                    help="start server automatically when live_server "
                         "fixture is applied (enabled by default).")
    group.addoption('--no-start-live-server',
                    action="store_false", dest="start_live_server",
                    help="don't start server automatically when live_server "
                         "fixture is applied.")
    group.addoption('--live-server-clean-stop',
                    action="store_true", dest="live_server_clean_stop", default=True,
                    help="attempt to kill the live server cleanly.")
    group.addoption('--no-live-server-clean-stop',
                    action="store_false", dest="live_server_clean_stop",
                    help="terminate the server forcefully after stop.")
    group.addoption('--live-server-host', action='store', default='localhost', type=str,
                    help='use a host where to listen (default localhost).')
    group.addoption('--live-server-port', action='store', default=0, type=int,
                    help='use a fixed port for the live_server fixture.')


def pytest_configure(config):
    config.addinivalue_line(
        'markers',
        'app(options): pass options to your application factory')
    config.addinivalue_line('markers', 'options: app config manipulation')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import pytest

from flask import request, url_for


class TestFixtures:

    def test_config_access(self, config):
        assert config['SECRET_KEY'] == '42'

    def test_client(self, client):
        assert client.get(url_for('ping')).status == '200 OK'

    def test_accept_json(self, accept_json):
        assert accept_json == [('Accept', 'application/json')]

    def test_accept_jsonp(self, accept_jsonp):
        assert accept_jsonp == [('Accept', 'application/json-p')]

    def test_request_ctx(self, app, request_ctx):
        assert request_ctx.app is app

    def test_request_ctx_is_kept_around(self, client):
        client.get(url_for('index'), headers=[('X-Something', '42')])
        assert request.headers['X-Something'] == '42'


class TestJSONResponse:

    def test_json_response(self, client, accept_json):
        res = client.get(url_for('ping'), headers=accept_json)
        assert res.json == {'ping': 'pong'}

    def test_json_response_compare_to_status_code(self, client, accept_json):
        assert client.get(url_for('ping'), headers=accept_json) == 200
        assert client.get('fake-route', headers=accept_json) == 404
        assert client.get('fake-route', headers=accept_json) != '404'
        res = client.get(url_for('ping'), headers=accept_json)
        assert res == res

    def test_mismatching_eq_comparison(self, client, accept_json):
        with pytest.raises(AssertionError, match=r'Mismatch in status code'):
            assert client.get('fake-route', headers=accept_json) == 200
        with pytest.raises(AssertionError, match=r'404 NOT FOUND'):
            assert client.get('fake-route', headers=accept_json) == '200'

    def test_dont_rewrite_existing_implementation(self, app, accept_json):
        class MyResponse(app.response_class):
            @property
            def json(self):
                """What is the meaning of life, the universe and everything?"""
                return 42

        app.response_class = MyResponse
        client = app.test_client()

        res = client.get(url_for('ping'), headers=accept_json)
        assert res.json == 42


@pytest.mark.usefixtures('client_class')
class TestClientClass:

    def test_client_attribute(self):
        assert hasattr(self, 'client')
        assert self.client.get(url_for('ping')).json == {'ping': 'pong'}
